==> cirru-writer-0.3.6.tgz: package/lib/writer.js <==
"use strict";

var util = require("./util"),
    transformer = require("./transformer"),
    bind = function bind(v, k) {
  return k(v);
},
    isString = function isString(x) {
  return typeof(x) === "string";
},
    isArray = function isArray(x) {
  return Array.isArray(x);
},
    tokenSegment = function tokenSegment(token) {
  return {
    type: "token",
    data: util.markToken(token)
  };
},
    textSegment = function textSegment(token) {
  return {
    type: "token",
    data: token
  };
},
    controlSegment = function controlSegment(op) {
  return {
    type: "control",
    data: op
  };
},
    renderInline = function renderInline(x) {
  return isString(x) ? util.markToken(x) : "(" + x.map(renderInline).join(" ") + ")";
},
    getTokenOrExpression = function getTokenOrExpression(x) {
  return isString(x) ? "string" : "expression";
},
    joinExpressions = function joinExpressions(acc, tree, lastType, isLastDeep) {
  return lastType === "start" ? joinExpressions(acc.concat(textSegment(renderInline(tree[0])), controlSegment("indent")), tree.slice(1), "string", false) : tree.length === 0 ? acc.concat(controlSegment("unindent")) : bind(tree[0], function (cursor) {
    return bind(getTokenOrExpression(cursor), function (thisType) {
      return joinExpressions(thisType === "string" ? lastType === "string" ? acc.concat(textSegment(" "), tokenSegment(cursor)) : acc.concat(controlSegment("newline"), tokenSegment(cursor)) : lastType === "string" && util.isSmallExpression(cursor) ? acc.concat(textSegment(" "), textSegment(renderInline(cursor))) : isLastDeep ? acc.concat(controlSegment("newline"), controlSegment("newline"), [handleExpression(cursor)]) : acc.concat(controlSegment("newline"), [handleExpression(cursor)]), tree.slice(1), thisType, util.isDeep(cursor));
    });
  });
},
    handleExpression = function handleExpression(tree) {
  return joinExpressions([], tree, "start", false);
},
    joinLines = function joinLines(acc, tree) {
  return tree.length === 0 ? acc : joinLines(acc.concat(controlSegment("newline"), tree.slice(0, 1), controlSegment("newline")), tree.slice(1));
},
    handleAST = function handleAST(tree) {
  var lines = tree.map(handleExpression);
  return joinLines([], lines);
},
    flattenSegments = function flattenSegments(acc, tree) {
  return tree.length === 0 ? acc : flattenSegments(acc.concat(bind(tree[0], function (cursor) {
    return isArray(cursor) ? flattenSegments([], cursor) : cursor;
  })), tree.slice(1));
},
    stringifySegments = function stringifySegments(acc, indentation, segments) {
  return segments.length === 0 ? acc : bind(segments[0], function (head) {
    return stringifySegments(acc + function () {
      switch (head.type) {
        case "token":
          return head.data;

        case "control":
          return function () {
            switch (head.data) {
              case "newline":
                return "\n" + indentation;

              default:
                return "";
            }
          }();

        default:
          return "";
      }
    }(), head.type === "control" ? function () {
      switch (head.data) {
        case "indent":
          return indentation + "  ";

        case "unindent":
          return indentation.slice(2);

        default:
          return indentation;
      }
    }() : indentation, segments.slice(1));
  });
};

exports.render = function (ast) {
  var fatAst = transformer.insertDollar(transformer.insertComma(ast)),
      segments = handleAST(fatAst),
      flattened = flattenSegments([], segments),
      result = stringifySegments("", "", flattened);
  return result.replace(new RegExp("\\n\\s+\\n", "g"), "\n\n");
};

==> cirru-writer-0.3.7.tgz: package/lib/writer.js <==
"use strict";

var util = require("./util"),
    transformer = require("./transformer"),
    bind = function bind(v, k) {
  return k(v);
},
    isString = function isString(x) {
  return typeof(x) === "string";
},
    isArray = function isArray(x) {
  return Array.isArray(x);
},
    tokenSegment = function tokenSegment(token) {
  return {
    type: "token",
    data: util.markToken(token)
  };
},
    textSegment = function textSegment(token) {
  return {
    type: "token",
    data: token
  };
},
    controlSegment = function controlSegment(op) {
  return {
    type: "control",
    data: op
  };
},
    renderInline = function renderInline(x) {
  return isString(x) ? util.markToken(x) : "(" + x.map(renderInline).join(" ") + ")";
},
    getTokenOrExpression = function getTokenOrExpression(x) {
  return isString(x) ? "string" : "expression";
},
    joinExpressions = function joinExpressions(acc, tree, lastType, isLastDeep, level) {
  return lastType === "start" ? isString(tree[0]) || util.isSmallExpression(tree[0]) || level < 1 ? joinExpressions(acc.concat(textSegment(renderInline(tree[0])), controlSegment("indent")), tree.slice(1), "string", false, level + 1) : joinExpressions(acc.concat(textSegment("  "), controlSegment("indent"), controlSegment("indent"), handleExpression(tree[0], level + 1), controlSegment("unindent")), tree.slice(1), "expression", false, level + 1) : tree.length === 0 ? acc.concat(controlSegment("unindent")) : bind(tree[0], function (cursor) {
    return bind(getTokenOrExpression(cursor), function (thisType) {
      return joinExpressions(thisType === "string" ? lastType === "string" ? acc.concat(textSegment(" "), tokenSegment(cursor)) : acc.concat(controlSegment("newline"), tokenSegment(cursor)) : lastType === "string" && util.isSmallExpression(cursor) ? acc.concat(textSegment(" "), textSegment(renderInline(cursor))) : isLastDeep ? acc.concat(controlSegment("newline"), controlSegment("newline"), [handleExpression(cursor, level + 1)]) : acc.concat(controlSegment("newline"), [handleExpression(cursor, level + 1)]), tree.slice(1), thisType, util.isDeep(cursor), level + 1);
    });
  });
},
    handleExpression = function handleExpression(tree, level) {
  return joinExpressions([], tree, "start", false, level);
},
    joinLines = function joinLines(acc, tree) {
  return tree.length === 0 ? acc : joinLines(acc.concat(controlSegment("newline"), tree.slice(0, 1), controlSegment("newline")), tree.slice(1));
},
    handleAST = function handleAST(tree) {
  var lines = tree.map(function (line) {
    return handleExpression(line, 0);
  });
  return joinLines([], lines);
},
    flattenSegments = function flattenSegments(acc, tree) {
  return tree.length === 0 ? acc : flattenSegments(acc.concat(bind(tree[0], function (cursor) {
    return isArray(cursor) ? flattenSegments([], cursor) : cursor;
  })), tree.slice(1));
},
    stringifySegments = function stringifySegments(acc, indentation, segments) {
  return segments.length === 0 ? acc : bind(segments[0], function (head) {
    return stringifySegments(acc + function () {
      switch (head.type) {
        case "token":
          return head.data;

        case "control":
          return function () {
            switch (head.data) {
              case "newline":
                return "\n" + indentation;

              default:
                return "";
            }
          }();

        default:
          return "";
      }
    }(), head.type === "control" ? function () {
      switch (head.data) {
        case "indent":
          return indentation + "  ";

        case "unindent":
          return indentation.slice(2);

        default:
          return indentation;
      }
    }() : indentation, segments.slice(1));
  });
};

exports.render = function (ast) {
  var fatAst = transformer.insertDollar(transformer.insertComma(ast)),
      segments = handleAST(fatAst),
      flattened = flattenSegments([], segments),
      result = stringifySegments("", "", flattened);
  return result.replace(new RegExp("\\n\\s+\\n", "g"), "\n\n");
};
